@@ -2,30 +2,32 @@ use strict;
 use warnings;
 package Email::Stuffer;
 # ABSTRACT: A more casual approach to creating and sending Email:: emails
-$Email::Stuffer::VERSION = '0.010'; # TRIAL
+$Email::Stuffer::VERSION = '0.015'; # TRIAL
+use Scalar::Util qw(blessed);
+
 #pod =head1 SYNOPSIS
 #pod
 #pod   # Prepare the message
 #pod   my $body = <<'AMBUSH_READY';
 #pod   Dear Santa
-#pod   
+#pod
 #pod   I have killed Bun Bun.
-#pod   
+#pod
 #pod   Yes, I know what you are thinking... but it was actually a total accident.
-#pod   
+#pod
 #pod   I was in a crowded line at a BayWatch signing, and I tripped, and stood on
 #pod   his head.
-#pod   
+#pod
 #pod   I know. Oops! :/
-#pod   
+#pod
 #pod   So anyways, I am willing to sell you the body for $1 million dollars.
-#pod   
+#pod
 #pod   Be near the pinhole to the Dimension of Pain at midnight.
-#pod   
+#pod
 #pod   Alias
 #pod
 #pod   AMBUSH_READY
-#pod   
+#pod
 #pod   # Create and send the email in one shot
 #pod   Email::Stuffer->from     ('cpan@ali.as'             )
 #pod                 ->to       ('santa@northpole.org'     )
@@ -89,7 +91,7 @@ $Email::Stuffer::VERSION = '0.010'; # TRIAL
 #pod   use Email::MIME;
 #pod   use Email::Sender::Simple;
 #pod   use IO::All;
-#pod   
+#pod
 #pod   Email::Sender::Simple->try_to_send(
 #pod     Email::MIME->create(
 #pod       header => [
@@ -128,7 +130,7 @@ $Email::Stuffer::VERSION = '0.010'; # TRIAL
 #pod
 #pod   package SMS::Alert;
 #pod   use base 'Email::Stuffer';
-#pod   
+#pod
 #pod   sub new {
 #pod     shift()->SUPER::new(@_)
 #pod            ->from('monitor@my.website')
@@ -166,9 +168,10 @@ use strict;
 use Carp                   qw(croak);
 use File::Basename         ();
 use Params::Util 1.05      qw(_INSTANCE _INSTANCEDOES);
-use Email::MIME            ();
+use Email::MIME 1.920      ();
 use Email::MIME::Creator   ();
 use Email::Sender::Simple  ();
+use Module::Runtime        qw(require_module);
 
 #####################################################################
 # Constructor and Accessors
@@ -177,25 +180,66 @@ use Email::Sender::Simple  ();
 #pod
 #pod Creates a new, empty, Email::Stuffer object.
 #pod
+#pod You can pass a hashref of properties to set, including:
+#pod
+#pod =for :list
+#pod * to
+#pod * from
+#pod * cc
+#pod * bcc
+#pod * reply_to
+#pod * subject
+#pod * text_body
+#pod * html_body
+#pod * transport
+#pod
+#pod The to, cc, bcc, and reply_to headers properties may be provided as array
+#pod references.  The array's contents will be used as the list of arguments to the
+#pod setter.
+#pod
 #pod =cut
 
+my %IS_INIT_ARG = map {; $_ => 1 } qw(
+  to from cc bcc reply_to subject text_body html_body transport
+);
+
+my %IS_ARRAY_ARG = map {; $_ => 1 } qw(
+  to cc bcc reply_to
+);
+
 sub new {
-	my $class = ref $_[0] || $_[0];
+  Carp::croak("new method called on Email::Stuffer instance") if ref $_[0];
 
-	my $self = bless {
-		parts      => [],
-		email      => Email::MIME->create(
-			header => [],
-			parts  => [],
-			),
-		}, $class;
+  my ($class, $arg) = @_;
 
-	$self;
+  my $self = bless {
+    parts      => [],
+    email      => Email::MIME->create(
+      header => [],
+      parts  => [],
+    ),
+  }, $class;
+
+  my @init_args = keys %{ $arg || {} };
+  if (my @bogus = grep {; ! $IS_INIT_ARG{$_} } @init_args) {
+    Carp::croak("illegal arguments to Email::Stuffer->new: @bogus");
+  }
+
+  for my $init_arg (@init_args) {
+    my @args = $arg->{$init_arg};
+    if ($IS_ARRAY_ARG{$init_arg} && ref $args[0] && ref $args[0] eq 'ARRAY') {
+      @args = @{ $args[0] };
+    }
+
+    $self->$init_arg(@args);
+  }
+
+  $self;
 }
 
 sub _self {
-	my $either = shift;
-	ref($either) ? $either : $either->new;
+  my $either = shift;
+  ref($either) ? $either : $either->new;
 }
 
 #pod =method header_names
@@ -206,11 +250,11 @@ sub _self {
 #pod =cut
 
 sub header_names {
-	shift()->{email}->header_names;
+  shift()->{email}->header_names;
 }
 
 sub headers {
-	shift()->{email}->header_names; ## This is now header_names, headers is depreciated
+  shift()->{email}->header_names; ## This is now header_names, headers is depreciated
 }
 
 #pod =method parts
@@ -220,7 +264,7 @@ sub headers {
 #pod =cut
 
 sub parts {
-	grep { defined $_ } @{shift()->{parts}};
+  grep { defined $_ } @{shift()->{parts}};
 }
 
 #####################################################################
@@ -234,21 +278,38 @@ sub parts {
 #pod =cut
 
 sub header {
-	my $self = shift()->_self;
-	return unless @_;
-	$self->{email}->header_str_set(ucfirst shift, shift);
-	return $self;
+  my $self = shift()->_self;
+  return unless @_;
+  $self->{email}->header_str_set(ucfirst shift, shift);
+  return $self;
 }
 
-#pod =method to $address
+#pod =method to @addresses
 #pod
 #pod Sets the To: header in the email
 #pod
 #pod =cut
 
+sub _assert_addr_list_ok {
+  my ($self, $header, $allow_empty, $list) = @_;
+
+  Carp::croak("$header is a required field")
+    unless $allow_empty or @$list;
+
+  for (@$list) {
+    Carp::croak("list of $header headers contains undefined values")
+      unless defined;
+
+    Carp::croak("list of $header headers contains unblessed references")
+      if ref && ! blessed $_;
+  }
+}
+
 sub to {
-	my $self = shift()->_self;
-	$self->{email}->header_str_set(To => join(q{, }, @_)) ? $self : undef;
+  my $self = shift()->_self;
+  $self->_assert_addr_list_ok(to => 0 => \@_);
+  $self->{email}->header_str_set(To => join(q{, }, @_));
+  return $self;
 }
 
 #pod =method from $address
@@ -258,30 +319,51 @@ sub to {
 #pod =cut
 
 sub from {
-	my $self = shift()->_self;
-	$self->{email}->header_str_set(From => shift) ? $self : undef;
+  my $self = shift()->_self;
+  $self->_assert_addr_list_ok(from => 0 => \@_);
+  Carp::croak("only one address is allowed in the from header") if @_ > 1;
+  $self->{email}->header_str_set(From => shift);
+  return $self;
 }
 
-#pod =method cc $address
+#pod =method reply_to $address
+#pod
+#pod Sets the Reply-To: header in the email
+#pod
+#pod =cut
+
+sub reply_to {
+  my $self = shift()->_self;
+  $self->_assert_addr_list_ok('reply-to' => 0 => \@_);
+  Carp::croak("only one address is allowed in the reply-to header") if @_ > 1;
+  $self->{email}->header_str_set('Reply-To' => shift);
+  return $self;
+}
+
+#pod =method cc @addresses
 #pod
 #pod Sets the Cc: header in the email
 #pod
 #pod =cut
 
 sub cc {
-	my $self = shift()->_self;
-	$self->{email}->header_str_set(Cc => join(q{, }, @_)) ? $self : undef;
+  my $self = shift()->_self;
+  $self->_assert_addr_list_ok(cc => 1 => \@_);
+  $self->{email}->header_str_set(Cc => join(q{, }, @_));
+  return $self;
 }
 
-#pod =method bcc $address
+#pod =method bcc @addresses
 #pod
 #pod Sets the Bcc: header in the email
 #pod
 #pod =cut
 
 sub bcc {
-	my $self = shift()->_self;
-	$self->{email}->header_str_set(Bcc => join(q{, }, @_)) ? $self : undef;
+  my $self = shift()->_self;
+  $self->_assert_addr_list_ok(bcc => 1 => \@_);
+  $self->{email}->header_str_set(Bcc => join(q{, }, @_));
+  return $self;
 }
 
 #pod =method subject $text
@@ -291,196 +373,214 @@ sub bcc {
 #pod =cut
 
 sub subject {
-	my $self = shift()->_self;
-	$self->{email}->header_str_set(Subject => shift) ? $self : undef;
+  my $self = shift()->_self;
+  Carp::croak("subject is a required field") unless defined $_[0];
+  $self->{email}->header_str_set(Subject => shift);
+  return $self;
 }
 
 #####################################################################
 # Body and Attachments
 
-#pod =method text_body $body [, $header => $value, ... ]
+#pod =method text_body $body [, $attribute => $value, ... ]
 #pod
-#pod Sets the text body of the email. Unless specified, all the appropriate
-#pod headers are set for you. You may override any as needed. See
-#pod L<Email::MIME> for the actual headers to use.
+#pod Sets the text body of the email. Appropriate headers are set for you.
+#pod You may override MIME attributes as needed. See the C<attributes>
+#pod parameter to L<Email::MIME/create> for the headers you can set.
 #pod
 #pod If C<$body> is undefined, this method will do nothing.
 #pod
+#pod Prior to Email::Stuffer version 0.015 text body was marked as flowed,
+#pod which broke all pre-formated body text.  Empty space at the beggining
+#pod of the line was dropped and every new line character could be changed
+#pod to one space (and vice versa).  Version 0.015 (and later) does not set
+#pod flowed format automatically anymore and so text body is really plain
+#pod text.  If you want to use old behavior of "advanced" flowed formatting,
+#pod set flowed format manually by: C<text_body($body, format => 'flowed')>.
+#pod
 #pod =cut
 
 sub text_body {
-	my $self = shift()->_self;
-	my $body = defined $_[0] ? shift : return $self;
-	my %attr = (
-		# Defaults
-		content_type => 'text/plain',
-		charset      => 'utf-8',
-		encoding     => 'quoted-printable',
-		format       => 'flowed',
-		# Params overwrite them
-		@_,
-		);
+  my $self = shift()->_self;
+  my $body = defined $_[0] ? shift : return $self;
+  my %attr = (
+    # Defaults
+    content_type => 'text/plain',
+    charset      => 'utf-8',
+    encoding     => 'quoted-printable',
+    # Params overwrite them
+    @_,
+    );
 
-	# Create the part in the text slot
-	$self->{parts}->[0] = Email::MIME->create(
-		attributes => \%attr,
-		body_str   => $body,
-		);
+  # Create the part in the text slot
+  $self->{parts}->[0] = Email::MIME->create(
+    attributes => \%attr,
+    body_str   => $body,
+    );
 
-	$self;
+  $self;
 }
 
 #pod =method html_body $body [, $header => $value, ... ]
 #pod
-#pod Set the HTML body of the email. Unless specified, all the appropriate
-#pod headers are set for you. You may override any as needed. See
-#pod L<Email::MIME> for the actual headers to use.
+#pod Sets the HTML body of the email. Appropriate headers are set for you.
+#pod You may override MIME attributes as needed. See the C<attributes>
+#pod parameter to L<Email::MIME/create> for the headers you can set.
 #pod
 #pod If C<$body> is undefined, this method will do nothing.
 #pod
 #pod =cut
 
 sub html_body {
-	my $self = shift()->_self;
-	my $body = defined $_[0] ? shift : return $self;
-	my %attr = (
-		# Defaults
-		content_type => 'text/html',
-		charset      => 'utf-8',
-		encoding     => 'quoted-printable',
-		# Params overwrite them
-		@_,
-		);
+  my $self = shift()->_self;
+  my $body = defined $_[0] ? shift : return $self;
+  my %attr = (
+    # Defaults
+    content_type => 'text/html',
+    charset      => 'utf-8',
+    encoding     => 'quoted-printable',
+    # Params overwrite them
+    @_,
+    );
 
-	# Create the part in the HTML slot
-	$self->{parts}->[1] = Email::MIME->create(
-		attributes => \%attr,
-		body_str   => $body,
-		);
+  # Create the part in the HTML slot
+  $self->{parts}->[1] = Email::MIME->create(
+    attributes => \%attr,
+    body_str   => $body,
+    );
 
-	$self;
+  $self;
 }
 
-#pod =method attach $contents [, $header => $value, ... ]
+#pod =method attach $contents [, $attribute => $value, ... ]
 #pod
 #pod Adds an attachment to the email. The first argument is the file contents
 #pod followed by (as for text_body and html_body) the list of headers to use.
-#pod Email::Stuffer should TRY to guess the headers right, but you may wish
-#pod to provide them anyway to be sure. Encoding is Base64 by default.
+#pod Email::Stuffer will I<try> to guess the headers correctly, but you may wish
+#pod to provide them anyway to be sure. Encoding is Base64 by default. See
+#pod the C<attributes> parameter to L<Email::MIME/create> for the headers you
+#pod can set.
 #pod
 #pod =cut
 
 sub _detect_content_type {
-	my ($filename, $body) = @_;
+  my ($filename, $body) = @_;
 
-	if (defined($filename)) {
-		if ($filename =~ /\.([a-z]{3,4})\z/) {
-			my $content_type = {
-				'gif'  => 'image/gif',
-				'png'  => 'image/png',
-				'jpg'  => 'image/jpeg',
-				'jpeg' => 'image/jpeg',
-				'txt'  => 'text/plain',
-				'htm'  => 'text/html',
-				'html' => 'text/html',
-				'css'  => 'text/css',
-			}->{$1};
-			return $content_type if defined $content_type;
-		}
-	}
-	if ($body =~ /
-		\A(?:
-		    (GIF8)          # gif
-		  | (\xff\xd8)      # jpeg
-		  | (\x89PNG)       # png
-		)
-	/x) {
-		return 'image/gif'  if $1;
-		return 'image/jpeg' if $2;
-		return 'image/png'  if $3;
-	}
-	return 'application/octet-stream';
+  if (defined($filename)) {
+    if ($filename =~ /\.([a-zA-Z]{3,4})\z/) {
+      my $content_type = {
+        'gif'  => 'image/gif',
+        'png'  => 'image/png',
+        'jpg'  => 'image/jpeg',
+        'jpeg' => 'image/jpeg',
+        'txt'  => 'text/plain',
+        'htm'  => 'text/html',
+        'html' => 'text/html',
+        'css'  => 'text/css',
+        'csv'  => 'text/csv',
+        'pdf'  => 'application/pdf',
+        'wav'  => 'audio/wav',
+      }->{lc($1)};
+      return $content_type if defined $content_type;
+    }
+  }
+  if ($body =~ /
+    \A(?:
+        (GIF8)          # gif
+      | (\xff\xd8)      # jpeg
+      | (\x89PNG)       # png
+      | (%PDF-)         # pdf
+    )
+  /x) {
+    return 'image/gif'  if $1;
+    return 'image/jpeg' if $2;
+    return 'image/png'  if $3;
+    return 'application/pdf' if $4;
+  }
+  return 'application/octet-stream';
 }
 
 sub attach {
-	my $self = shift()->_self;
-	my $body = defined $_[0] ? shift : return undef;
-	my %attr = (
-		# Cheap defaults
-		encoding => 'base64',
-		# Params overwrite them
-		@_,
-		);
+  my $self = shift()->_self;
+  my $body = defined $_[0] ? shift : return undef;
+  my %attr = (
+    # Cheap defaults
+    encoding => 'base64',
+    # Params overwrite them
+    @_,
+    );
 
-	# The more expensive defaults if needed
-	unless ( $attr{content_type} ) {
-		$attr{content_type} = _detect_content_type($attr{filename}, $body);
-	}
+  # The more expensive defaults if needed
+  unless ( $attr{content_type} ) {
+    $attr{content_type} = _detect_content_type($attr{filename}, $body);
+  }
 
-	### MORE?
+  ### MORE?
 
-	# Determine the slot to put it at
-	my $slot = scalar @{$self->{parts}};
-	$slot = 3 if $slot < 3;
+  # Determine the slot to put it at
+  my $slot = scalar @{$self->{parts}};
+  $slot = 3 if $slot < 3;
 
-	# Create the part in the attachment slot
-	$self->{parts}->[$slot] = Email::MIME->create(
-		attributes => \%attr,
-		body       => $body,
-		);
+  # Create the part in the attachment slot
+  $self->{parts}->[$slot] = Email::MIME->create(
+    attributes => \%attr,
+    body       => $body,
+    );
 
-	$self;
+  $self;
 }
 
-#pod =method attach_file $file [, $header => $value, ... ]
+#pod =method attach_file $file [, $attribuet => $value, ... ]
 #pod
-#pod Attachs a file that already exists on the filesystem to the email. 
-#pod C<attach_file> will auto-detect the MIME type, and use the file's
-#pod current name when attaching.
+#pod Attachs a file that already exists on the filesystem to the email.
+#pod C<attach_file> will attempt to auto-detect the MIME type, and use the
+#pod file's current name when attaching. See the C<attributes> parameter to
+#pod L<Email::MIME/create> for the headers you can set.
 #pod
 #pod =cut
 
 sub attach_file {
-	my $self = shift;
-	my $body_arg = shift;
-	my $name = undef;
-	my $body = undef;
+  my $self = shift;
+  my $body_arg = shift;
+  my $name = undef;
+  my $body = undef;
 
-	# Support IO::All::File arguments
-	if ( Params::Util::_INSTANCE($body_arg, 'IO::All::File') ) {
-		$name = $body_arg->name;
-		$body = $body_arg->all;
+  # Support IO::All::File arguments
+  if ( Params::Util::_INSTANCE($body_arg, 'IO::All::File') ) {
+    $body_arg->binmode;
+    $name = $body_arg->name;
+    $body = $body_arg->all;
 
-	# Support file names
-	} elsif ( defined $body_arg and Params::Util::_STRING($body_arg) ) {
-		croak "No such file '$body_arg'" unless -f $body_arg;
-		$name = $body_arg;
-		$body = _slurp( $body_arg );
+  # Support file names
+  } elsif ( defined $body_arg and Params::Util::_STRING($body_arg) ) {
+    croak "No such file '$body_arg'" unless -f $body_arg;
+    $name = $body_arg;
+    $body = _slurp( $body_arg );
 
-	# That's it
-	} else {
-		my $type = ref($body_arg) || "<$body_arg>";
-		croak "Expected a file name or an IO::All::File derivative, got $type";
-	}
+  # That's it
+  } else {
+    my $type = ref($body_arg) || "<$body_arg>";
+    croak "Expected a file name or an IO::All::File derivative, got $type";
+  }
 
-	# Clean the file name
-	$name = File::Basename::basename($name);
+  # Clean the file name
+  $name = File::Basename::basename($name);
 
-	croak("basename somehow returned undef") unless defined $name;
+  croak("basename somehow returned undef") unless defined $name;
 
-	# Now attach as normal
-	$self->attach( $body, name => $name, filename => $name, @_ );
+  # Now attach as normal
+  $self->attach( $body, name => $name, filename => $name, @_ );
 }
 
 # Provide a simple _slurp implementation
 sub _slurp {
-	my $file = shift;
-	local $/ = undef;
+  my $file = shift;
+  local $/ = undef;
 
-	open my $slurp, '<:raw', $file or croak("error opening $file: $!");
-	my $source = <$slurp>;
-	close( $slurp ) or croak "error after slurping $file: $!";
-	\$source;
+  open my $slurp, '<:raw', $file or croak("error opening $file: $!");
+  my $source = <$slurp>;
+  close( $slurp ) or croak "error after slurping $file: $!";
+  \$source;
 }
 
 #pod =method transport
@@ -505,23 +605,24 @@ sub _slurp {
 #pod =cut
 
 sub transport {
-	my $self = shift;
+  my $self = shift;
 
-	if ( @_ ) {
-		# Change the transport
-		if ( _INSTANCEDOES($_[0], 'Email::Sender::Transport') ) {
-			$self->{transport} = shift;
-		} else {
-		  my ($moniker, @arg) = @_;
-		  my $class = $moniker =~ s/\A=//
-		            ? $moniker
-		            : "Email::Sender::Transport::$moniker";
-			my $transport = $class->new(@arg);
-			$self->{transport} = $transport;
-		}
-	}
+  if ( @_ ) {
+    # Change the transport
+    if ( _INSTANCEDOES($_[0], 'Email::Sender::Transport') ) {
+      $self->{transport} = shift;
+    } else {
+      my ($moniker, @arg) = @_;
+      my $class = $moniker =~ s/\A=//
+                ? $moniker
+                : "Email::Sender::Transport::$moniker";
+      require_module($class);
+      my $transport = $class->new(@arg);
+      $self->{transport} = $transport;
+    }
+  }
 
-	$self;
+  $self;
 }
 
 #####################################################################
@@ -534,30 +635,32 @@ sub transport {
 #pod =cut
 
 sub email {
-	my $self  = shift;
-	my @parts = $self->parts;
+  my $self  = shift;
+  my @parts = $self->parts;
 
-        ### Lyle Hopkins, code added to Fix single part, and multipart/alternative problems
-        if ( scalar( @{ $self->{parts} } ) >= 3 ) {
-                ## multipart/mixed
-                $self->{email}->parts_set( \@parts );
-        }
-        ## Check we actually have any parts
-        elsif ( scalar( @{ $self->{parts} } ) ) {
-                if ( _INSTANCE($parts[0], 'Email::MIME') && _INSTANCE($parts[1], 'Email::MIME') ) {
-                        ## multipart/alternate
-                        $self->{email}->header_set( 'Content-Type' => 'multipart/alternative' );
-                        $self->{email}->parts_set( \@parts );
-                }
-                ## As @parts is $self->parts without the blanks, we only need check $parts[0]
-                elsif ( _INSTANCE($parts[0], 'Email::MIME') ) {
-                        ## single part text/plain
-                        _transfer_headers( $self->{email}, $parts[0] );
-                        $self->{email} = $parts[0];
-                }
-        }
+  ### Lyle Hopkins, code added to Fix single part, and multipart/alternative
+  ### problems
+  if (scalar(@{ $self->{parts} }) >= 3) {
+    ## multipart/mixed
+    $self->{email}->parts_set(\@parts);
+  } elsif (scalar(@{ $self->{parts} })) {
+    ## Check we actually have any parts
+    if ( _INSTANCE($parts[0], 'Email::MIME')
+      && _INSTANCE($parts[1], 'Email::MIME')
+    ) {
+      ## multipart/alternate
+      $self->{email}->header_set('Content-Type' => 'multipart/alternative');
+      $self->{email}->parts_set(\@parts);
+    } elsif (_INSTANCE($parts[0], 'Email::MIME')) {
+      ## As @parts is $self->parts without the blanks, we only need check
+      ## $parts[0]
+      ## single part text/plain
+      _transfer_headers($self->{email}, $parts[0]);
+      $self->{email} = $parts[0];
+    }
+  }
 
-	$self->{email};
+  $self->{email};
 }
 
 # Support coercion to an Email::MIME
@@ -593,7 +696,7 @@ sub _transfer_headers {
 #pod =cut
 
 sub as_string {
-	shift()->email->as_string;
+  shift()->email->as_string;
 }
 
 #pod =method send
@@ -605,15 +708,15 @@ sub as_string {
 #pod =cut
 
 sub send {
-	my $self = shift;
-	my $arg  = shift;
-	my $email = $self->email or return undef;
+  my $self = shift;
+  my $arg  = shift;
+  my $email = $self->email or return undef;
 
-	my $transport = $self->{transport};
+  my $transport = $self->{transport};
 
-	Email::Sender::Simple->try_to_send(
-	  $email,
-	  {
+  Email::Sender::Simple->try_to_send(
+    $email,
+    {
       ($transport ? (transport => $transport) : ()),
       $arg ? %$arg : (),
     },
@@ -629,15 +732,15 @@ sub send {
 #pod =cut
 
 sub send_or_die {
-	my $self = shift;
-	my $arg  = shift;
-	my $email = $self->email or return undef;
+  my $self = shift;
+  my $arg  = shift;
+  my $email = $self->email or return undef;
 
-	my $transport = $self->{transport};
+  my $transport = $self->{transport};
 
-	Email::Sender::Simple->send(
-	  $email,
-	  {
+  Email::Sender::Simple->send(
+    $email,
+    {
       ($transport ? (transport => $transport) : ()),
       $arg ? %$arg : (),
     },
@@ -671,31 +774,31 @@ Email::Stuffer - A more casual approach to creating and sending Email:: emails
 
 =head1 VERSION
 
-version 0.010
+version 0.015
 
 =head1 SYNOPSIS
 
   # Prepare the message
   my $body = <<'AMBUSH_READY';
   Dear Santa
-  
+
   I have killed Bun Bun.
-  
+
   Yes, I know what you are thinking... but it was actually a total accident.
-  
+
   I was in a crowded line at a BayWatch signing, and I tripped, and stood on
   his head.
-  
+
   I know. Oops! :/
-  
+
   So anyways, I am willing to sell you the body for $1 million dollars.
-  
+
   Be near the pinhole to the Dimension of Pain at midnight.
-  
+
   Alias
 
   AMBUSH_READY
-  
+
   # Create and send the email in one shot
   Email::Stuffer->from     ('cpan@ali.as'             )
                 ->to       ('santa@northpole.org'     )
@@ -759,7 +862,7 @@ what the correct MIME structure is.
   use Email::MIME;
   use Email::Sender::Simple;
   use IO::All;
-  
+
   Email::Sender::Simple->try_to_send(
     Email::MIME->create(
       header => [
@@ -791,10 +894,64 @@ Email:: modules underneath.
 
 =head1 METHODS
 
+As you can see from the synopsis, all methods that B<modify> the
+Email::Stuffer object returns the object, and thus most normal calls are
+chainable.
+
+However, please note that C<send>, and the group of methods that do not
+change the Email::Stuffer object B<do not> return the object, and thus
+B<are not> chainable.
+
 =head2 new
 
 Creates a new, empty, Email::Stuffer object.
 
+You can pass a hashref of properties to set, including:
+
+=over 4
+
+=item *
+
+to
+
+=item *
+
+from
+
+=item *
+
+cc
+
+=item *
+
+bcc
+
+=item *
+
+reply_to
+
+=item *
+
+subject
+
+=item *
+
+text_body
+
+=item *
+
+html_body
+
+=item *
+
+transport
+
+=back
+
+The to, cc, bcc, and reply_to headers properties may be provided as array
+references.  The array's contents will be used as the list of arguments to the
+setter.
+
 =head2 header_names
 
 Returns, as a list, all of the headers currently set for the Email
@@ -809,7 +966,7 @@ Returns, as a list, the L<Email::MIME> parts for the Email
 Sets a named header in the email. Multiple calls with the same $header
 will overwrite previous calls $value.
 
-=head2 to $address
+=head2 to @addresses
 
 Sets the To: header in the email
 
@@ -817,11 +974,15 @@ Sets the To: header in the email
 
 Sets the From: header in the email
 
-=head2 cc $address
+=head2 reply_to $address
+
+Sets the Reply-To: header in the email
+
+=head2 cc @addresses
 
 Sets the Cc: header in the email
 
-=head2 bcc $address
+=head2 bcc @addresses
 
 Sets the Bcc: header in the email
 
@@ -829,34 +990,45 @@ Sets the Bcc: header in the email
 
 Sets the Subject: header in the email
 
-=head2 text_body $body [, $header => $value, ... ]
+=head2 text_body $body [, $attribute => $value, ... ]
 
-Sets the text body of the email. Unless specified, all the appropriate
-headers are set for you. You may override any as needed. See
-L<Email::MIME> for the actual headers to use.
+Sets the text body of the email. Appropriate headers are set for you.
+You may override MIME attributes as needed. See the C<attributes>
+parameter to L<Email::MIME/create> for the headers you can set.
 
 If C<$body> is undefined, this method will do nothing.
 
+Prior to Email::Stuffer version 0.015 text body was marked as flowed,
+which broke all pre-formated body text.  Empty space at the beggining
+of the line was dropped and every new line character could be changed
+to one space (and vice versa).  Version 0.015 (and later) does not set
+flowed format automatically anymore and so text body is really plain
+text.  If you want to use old behavior of "advanced" flowed formatting,
+set flowed format manually by: C<text_body($body, format => 'flowed')>.
+
 =head2 html_body $body [, $header => $value, ... ]
 
-Set the HTML body of the email. Unless specified, all the appropriate
-headers are set for you. You may override any as needed. See
-L<Email::MIME> for the actual headers to use.
+Sets the HTML body of the email. Appropriate headers are set for you.
+You may override MIME attributes as needed. See the C<attributes>
+parameter to L<Email::MIME/create> for the headers you can set.
 
 If C<$body> is undefined, this method will do nothing.
 
-=head2 attach $contents [, $header => $value, ... ]
+=head2 attach $contents [, $attribute => $value, ... ]
 
 Adds an attachment to the email. The first argument is the file contents
 followed by (as for text_body and html_body) the list of headers to use.
-Email::Stuffer should TRY to guess the headers right, but you may wish
-to provide them anyway to be sure. Encoding is Base64 by default.
+Email::Stuffer will I<try> to guess the headers correctly, but you may wish
+to provide them anyway to be sure. Encoding is Base64 by default. See
+the C<attributes> parameter to L<Email::MIME/create> for the headers you
+can set.
 
-=head2 attach_file $file [, $header => $value, ... ]
+=head2 attach_file $file [, $attribuet => $value, ... ]
 
-Attachs a file that already exists on the filesystem to the email. 
-C<attach_file> will auto-detect the MIME type, and use the file's
-current name when attaching.
+Attachs a file that already exists on the filesystem to the email.
+C<attach_file> will attempt to auto-detect the MIME type, and use the
+file's current name when attaching. See the C<attributes> parameter to
+L<Email::MIME/create> for the headers you can set.
 
 =head2 transport
 
@@ -907,7 +1079,7 @@ Email::Stuffer's brevity to your advantage.
 
   package SMS::Alert;
   use base 'Email::Stuffer';
-  
+
   sub new {
     shift()->SUPER::new(@_)
            ->from('monitor@my.website')
@@ -928,16 +1100,6 @@ Z<>
                     ->send;
   }
 
-=head1 METHODS
-
-As you can see from the synopsis, all methods that B<modify> the
-Email::Stuffer object returns the object, and thus most normal calls are
-chainable.
-
-However, please note that C<send>, and the group of methods that do not
-change the Email::Stuffer object B<do not> return the object, and thus
-B<are not> chainable.
-
 =head1 TO DO
 
 =over 4
@@ -974,6 +1136,98 @@ Ricardo SIGNES <rjbs@cpan.org>
 
 =back
 
+=head1 CONTRIBUTORS
+
+=for stopwords Aaron W. Swenson adam adamk@cpan.org adam@phase-n.com Aristotle Pagaltzis Arthur Axel 'fREW' Schmidt Chase Whitener CosmicNet Dan Book John Napiorkowski Josh Stompro Kevin Tew Kieren Diment Kris Matthews Lee Johnson Manni Heumann Pali Ross Attrill Shawn Sorichetti tokuhirom
+
+=over 4
+
+=item *
+
+Aaron W. Swenson <aaron.w.swenson@gmail.com>
+
+=item *
+
+adam <adam@88f4d9cd-8a04-0410-9d60-8f63309c3137>
+
+=item *
+
+adamk@cpan.org <adamk@cpan.org@88f4d9cd-8a04-0410-9d60-8f63309c3137>
+
+=item *
+
+adam@phase-n.com <adam@phase-n.com@88f4d9cd-8a04-0410-9d60-8f63309c3137>
+
+=item *
+
+Aristotle Pagaltzis <pagaltzis@gmx.de>
+
+=item *
+
+Arthur Axel 'fREW' Schmidt <frioux@gmail.com>
+
+=item *
+
+Chase Whitener <chase.whitener@infotechfl.com>
+
+=item *
+
+CosmicNet <webmaster@cosmicperl.com>
+
+=item *
+
+Dan Book <grinnz@gmail.com>
+
+=item *
+
+John Napiorkowski <jjn1056@yahoo.com>
+
+=item *
+
+Josh Stompro <github@stompro.org>
+
+=item *
+
+Kevin Tew <tewk@tan.tewk.com>
+
+=item *
+
+Kieren Diment <kd@fenchurch.local>
+
+=item *
+
+Kris Matthews <krismatth@icloud.com>
+
+=item *
+
+Kris Matthews <kris@tigerlms.com>
+
+=item *
+
+Lee Johnson <lee@givengain.ch>
+
+=item *
+
+Manni Heumann <github@lxxi.org>
+
+=item *
+
+Pali <pali@cpan.org>
+
+=item *
+
+Ross Attrill <ross.attrill@gmail.com>
+
+=item *
+
+Shawn Sorichetti <shawn@coloredblocks.com>
+
+=item *
+
+tokuhirom <tokuhirom@gmail.com>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2004 by Adam Kennedy and Ricardo SIGNES.
